/*!
        @file    pyb_gradientflow.cpp

        @brief   python binding of gradient flow classes

        @author  Tatsumi Aoyama (aoym)
                 $LastChangedBy: aoyama $

        @date    $LastChangedDate:: 2021-02-25 17:32:18 #$

        @version $LastChangedRevision: 2177 $
*/

#include <tuple>
#include "pyb_defs.h"
#include "pyb_params.h"
#include "pyb_gradientflow.h"
#include "bridge.h"
#include "Field/field.h"
#include "Measurements/Gauge/gradientFlow.h"
#include "Parameters/parameters.h"
#include "pyb_wrap.h"




void pybind_gradientflow(py::module m)
{
  //! GradientFlow
  //
  py::class_<GradientFlow>(m, "GradientFlow", R"pybdoc(
Gradient flow class

Parameters
----------
order_of_RungeKutta : int
step_size : float
order_of_approx_for_exp_iP : int
adaptive : {'true', 'false'}
tolerance : float, mandatory if adaptive=true
safety_factor : float, mandatory if adaptive=true
)pybdoc")
    .def(py::init([](py::object action, py::kwargs kwargs) -> GradientFlow*
                  {
                    auto obj = new PyWrap1<GradientFlow, Action>(action);

                    if (obj) obj->set_parameters(make_parameters(kwargs));
                    return obj;
                  }), REDIRECT)

    .def("set_parameters", [](GradientFlow& self, py::kwargs kwargs)
         {
           self.set_parameters(make_parameters(kwargs));
         }, REDIRECT)

    .def("evolve", [](GradientFlow& self, double t, Field_G& U) -> std::tuple<double, double>
         {
           double tt = t;
           double r = self.evolve(tt, U);
           return std::tuple<double,double>(tt, r);
         }, REDIRECT)
    ;


}
